## Section Convolution
import numpy as np
import math


def p_point_dft(x, p):

    x = np.asarray(x, dtype=float)
    N = len(x)
    
    # Zero pad x if p > N
    if p > N:
        x = np.pad(x, (0, p - N), mode='constant')
    else:
        x = x[:p]
    
    n = np.arange(p)
    k = n.reshape((p, 1))
    M = np.exp(-2j * np.pi * k * n / p)
    return np.dot(M, x)

def p_point_idft(X, p):

    X = np.asarray(X, dtype=complex)
    N = len(X)
    
    # Zero pad X if p > N
    if p > N:
        X = np.pad(X, (0, p - N), mode='constant')
    else:
        X = X[:p]
    
    n = np.arange(p)
    k = n.reshape((p, 1))
    M = np.exp(2j * np.pi * k * n / p)
    return np.dot(M, X) / p


def conv1d(x, h):

    N = len(x)
    M = len(h)
    L = N + M - 1
    x = np.pad(x, (0, L-N), 'constant')
    h = np.pad(h, (0, L-M), 'constant')
    y = p_point_idft(p_point_dft(x, L) * p_point_dft(h, L), L)
    return y

def get_section(x, nth_section, L):
    return x[nth_section*L:(nth_section+1)*L]

def merge_section(y, x, nth_section, L):
    for i in range(len(y)):
        idx = i - nth_section*L
        if idx >= 0 and idx < len(x):
            y[i] += x[idx]
    return y

def section_conv(x, h, point, L):

    N = len(x)
    M = len(h)
    S = int(math.ceil(N/L))
    # res = np.zeros()
    res = np.zeros(N+M-1, dtype=complex)
    res_golden = np.convolve(x, h, 'full')

    for i in range(S):
        x_section = get_section(x, i, L)
        h_section = h
        y = p_point_idft(p_point_dft(x_section, point) * p_point_dft(h_section, point), point)
        # print(y_golden)
        # print(y, y_golden)
        res = merge_section(res, y, i, L)
        # res_golden = np.concatenate((res_golden, y_golden))
    res = np.real(res)
    return res, res_golden



if __name__ == "__main__":
    x = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16])
    h = np.array([1, 2, 3, 4])
    point = 8
    L = 4
    res, res_golden = section_conv(x, h, point, L)
    print("Result of Section Conv:", res)
    print("Golden", res_golden)
    # print(p_point_idft(p_point_dft(h, 4), 4))
    # Define the input signal and the kernel
    # x = np.array([1, 2, 3, 4, 5])
    # h = np.array([1, 0, -1])

    # # Perform the convolution
    # y_full = np.convolve(x, h, mode='full')
    # y_valid = np.convolve(x, h, mode='valid')
    # y_same = np.convolve(x, h, mode='same')

    # print("Input signal:", x)
    # print("Kernel:", h)
    # print("Full convolution result:", y_full)
    # print("Valid convolution result:", y_valid)
    # print("Same convolution result:", y_same)